@@ -67,7 +67,7 @@ mousepad_dialogs_show_about (GtkWindow *parent)
   /* show the dialog */
   gtk_show_about_dialog (parent,
                          "authors", authors,
-                         "comments", _("Mousepad is a fast text editor for the Xfce Desktop Environment."),
+                         "comments", _("Mousepad is a simple text editor for the Xfce desktop environment"),
                          "copyright", "Copyright \xc2\xa9 2005-2021 - the Mousepad developers",
                          "destroy-with-parent", TRUE,
                          "license-type", GTK_LICENSE_GPL_2_0,
@@ -849,7 +849,7 @@ mousepad_dialogs_combo_changed (GtkComboBox *combo,
           if ((files = gtk_file_chooser_get_files (GTK_FILE_CHOOSER (dialog))) != NULL)
             {
               file = mousepad_file_new (GTK_TEXT_BUFFER (gtk_source_buffer_new (NULL)));
-              mousepad_file_set_location (file, files->data, TRUE);
+              mousepad_file_set_location (file, files->data, MOUSEPAD_LOCATION_REAL);
               mousepad_file_set_encoding (file, encoding);
               g_slist_free_full (files, g_object_unref);
             }
@@ -870,7 +870,7 @@ mousepad_dialogs_combo_changed (GtkComboBox *combo,
           if ((g_file = g_file_new_tmp (NULL, &iostream, &error)) != NULL)
             {
               file = mousepad_file_new (mousepad_file_get_buffer (current_file));
-              mousepad_file_set_location (file, g_file, TRUE);
+              mousepad_file_set_location (file, g_file, MOUSEPAD_LOCATION_REAL);
               if (encoding != MOUSEPAD_ENCODING_NONE)
                 mousepad_file_set_encoding (file, encoding);
               else
@@ -1222,7 +1222,7 @@ mousepad_dialogs_open (GtkWindow         *parent,
                     G_CALLBACK (mousepad_dialogs_open_selection_changed), combo);
 
   /* select the active document in the file chooser */
-  if (file != NULL && g_file_query_exists (file, NULL))
+  if (file != NULL && mousepad_util_query_exists (file, FALSE))
     gtk_file_chooser_set_file (GTK_FILE_CHOOSER (dialog), file, NULL);
 
   /* run the dialog */
