@@ -36,14 +36,6 @@ static void      mousepad_document_notify_language         (GtkSourceBuffer
 static void      mousepad_document_notify_overwrite        (GtkTextView            *textview,
                                                             GParamSpec             *pspec,
                                                             MousepadDocument       *document);
-static void      mousepad_document_drag_data_received      (GtkWidget              *widget,
-                                                            GdkDragContext         *context,
-                                                            gint                    x,
-                                                            gint                    y,
-                                                            GtkSelectionData       *selection_data,
-                                                            guint                   info,
-                                                            guint                   drag_time,
-                                                            MousepadDocument       *document);
 static void      mousepad_document_location_changed        (MousepadDocument       *document,
                                                             GFile                  *file);
 static void      mousepad_document_label_color             (MousepadDocument       *document);
@@ -101,6 +93,7 @@ struct _MousepadDocumentPrivate
   GtkSourceSearchContext *search_context, *selection_context;
   GtkSourceBuffer        *selection_buffer;
   gint                    prev_search_state;
+  guint                   search_id;
 };
 
 
@@ -241,6 +234,7 @@ mousepad_document_init (MousepadDocument *document)
   document->buffer = GTK_TEXT_BUFFER (gtk_source_buffer_new (NULL));
   document->priv->search_context = gtk_source_search_context_new (
                                      GTK_SOURCE_BUFFER (document->buffer), NULL);
+  document->priv->search_id = 0;
 
   /* bind search settings to Mousepad settings, except "regex-enabled" to prevent prohibitive
    * computation times in some situations (see
@@ -278,8 +272,6 @@ mousepad_document_init (MousepadDocument *document)
                             G_CALLBACK (mousepad_document_label_color), document);
   g_signal_connect_swapped (document->textview, "notify::editable",
                             G_CALLBACK (mousepad_document_label_color), document);
-  g_signal_connect (document->textview, "drag-data-received",
-                    G_CALLBACK (mousepad_document_drag_data_received), document);
 
   /* forward some document attribute signals more or less directly */
   g_signal_connect_swapped (document->buffer, "notify::cursor-position",
@@ -296,56 +288,6 @@ mousepad_document_init (MousepadDocument *document)
 
 
 
-static gboolean
-mousepad_document_unref (gpointer data)
-{
-  g_object_unref (data);
-
-  return FALSE;
-}
-
-
-
-static void
-mousepad_document_post_finalize (GtkSourceSearchContext *search_context,
-                                 GParamSpec             *pspec,
-                                 gpointer                buffer)
-{
-  /* let the search context finish its work completely (no warning without that,
-   * but Valgrind sees invalid reads) */
-  g_idle_add (mousepad_document_unref, search_context);
-  g_idle_add (mousepad_document_unref, buffer);
-}
-
-
-
-static void
-mousepad_document_finalize_search (MousepadDocument       *document,
-                                   gpointer                buffer,
-                                   GtkSourceSearchContext *search_context)
-{
-  GtkSourceSearchSettings *search_settings;
-
-  /* disconnect any handler connected to the buffer or the search context, to not interfer
-   * with what follows */
-  g_signal_handlers_disconnect_by_data (search_context, document);
-  g_signal_handlers_disconnect_by_data (buffer, document);
-
-  /* reset some critical settings to ensure that the last search is almost instantaneous
-   * (see mousepad_document_prevent_endless_scanning() below) */
-  search_settings = gtk_source_search_context_get_settings (search_context);
-  gtk_source_search_context_set_highlight (search_context, FALSE);
-  gtk_source_search_settings_set_regex_enabled (search_settings, FALSE);
-
-  /* cancel any current search by launching a last one, the result of which will trigger
-   * post_finalize() */
-  g_signal_connect (search_context, "notify::occurrences-count",
-                    G_CALLBACK (mousepad_document_post_finalize), buffer);
-  gtk_source_search_settings_set_search_text (search_settings, NULL);
-}
-
-
-
 static void
 mousepad_document_finalize (GObject *object)
 {
@@ -359,16 +301,16 @@ mousepad_document_finalize (GObject *object)
   /* release the file */
   g_object_unref (document->file);
 
-  /*
-   * We will release buffers and search contexts when a last search is completed,
-   * including buffers scanning, to prevent too late accesses to the buffers.
-   */
-  mousepad_document_finalize_search (document, document->buffer, document->priv->search_context);
+  /* search related */
+  g_object_unref (document->priv->search_context);
+  g_object_unref (document->buffer);
   if (document->priv->selection_buffer != NULL)
-    mousepad_document_finalize_search (document, document->priv->selection_buffer,
-                                       document->priv->selection_context);
+    {
+      g_object_unref (document->priv->selection_context);
+      g_object_unref (document->priv->selection_buffer);
+    }
 
-  (*G_OBJECT_CLASS (mousepad_document_parent_class)->finalize) (object);
+  G_OBJECT_CLASS (mousepad_document_parent_class)->finalize (object);
 }
 
 
@@ -472,26 +414,6 @@ mousepad_document_send_signals (MousepadDocument *document)
 
 
 
-static void
-mousepad_document_drag_data_received (GtkWidget        *widget,
-                                      GdkDragContext   *context,
-                                      gint              x,
-                                      gint              y,
-                                      GtkSelectionData *selection_data,
-                                      guint             info,
-                                      guint             drag_time,
-                                      MousepadDocument *document)
-{
-  g_return_if_fail (MOUSEPAD_IS_DOCUMENT (document));
-
-  /* emit the drag-data-received signal from the document when a tab or uri has been dropped */
-  if (info == TARGET_TEXT_URI_LIST || info == TARGET_GTK_NOTEBOOK_TAB)
-    g_signal_emit_by_name (document, "drag-data-received", context,
-                           x, y, selection_data, info, drag_time);
-}
-
-
-
 static void
 mousepad_document_location_changed (MousepadDocument *document,
                                     GFile            *file)
@@ -503,51 +425,40 @@ mousepad_document_location_changed (MousepadDocument *document,
   g_return_if_fail (MOUSEPAD_IS_DOCUMENT (document));
   g_return_if_fail (file != NULL);
 
-  /* convert the title into a utf-8 valid version for display */
-  utf8_filename = (gchar *) mousepad_util_get_path (file);
-  if (G_LIKELY (utf8_filename != NULL))
-    utf8_filename = g_filename_to_utf8 (utf8_filename, -1, NULL, NULL, NULL);
-  else
+  /* get the display filename */
+  utf8_filename = mousepad_util_get_display_path (file);
+
+  /* create a shorter display filename: replace $HOME with a tilde if user is not root */
+  if (geteuid () && (home = g_get_home_dir ()) && (home_len = strlen (home))
+      && g_str_has_prefix (utf8_filename, home))
     {
-      utf8_short_filename = g_file_get_uri (file);
-      utf8_filename = g_uri_unescape_string (utf8_short_filename, NULL);
-      g_free (utf8_short_filename);
+      utf8_short_filename = g_strconcat ("~", utf8_filename + home_len, NULL);
+      g_free (utf8_filename);
+      utf8_filename = utf8_short_filename;
     }
 
-  if (G_LIKELY (utf8_filename))
+  /* create the display name */
+  utf8_basename = g_filename_display_basename (utf8_filename);
+
+  /* remove the old names */
+  g_free (document->priv->utf8_filename);
+  g_free (document->priv->utf8_basename);
+
+  /* set the new names */
+  document->priv->utf8_filename = utf8_filename;
+  document->priv->utf8_basename = utf8_basename;
+
+  /* update the tab label and tooltip */
+  if (G_UNLIKELY (document->priv->label))
     {
-      /* create a shorter display filename: replace $HOME with a tilde if user is not root */
-      if (geteuid () && (home = g_get_home_dir ()) && (home_len = strlen (home))
-          && g_str_has_prefix (utf8_filename, home))
-        {
-          utf8_short_filename = g_strconcat ("~", utf8_filename + home_len, NULL);
-          g_free (utf8_filename);
-          utf8_filename = utf8_short_filename;
-        }
+      /* set the tab label */
+      gtk_label_set_text (GTK_LABEL (document->priv->label), utf8_basename);
 
-      /* create the display name */
-      utf8_basename = g_filename_display_basename (utf8_filename);
+      /* set the tab tooltip */
+      gtk_widget_set_tooltip_text (document->priv->ebox, utf8_filename);
 
-      /* remove the old names */
-      g_free (document->priv->utf8_filename);
-      g_free (document->priv->utf8_basename);
-
-      /* set the new names */
-      document->priv->utf8_filename = utf8_filename;
-      document->priv->utf8_basename = utf8_basename;
-
-      /* update the tab label and tooltip */
-      if (G_UNLIKELY (document->priv->label))
-        {
-          /* set the tab label */
-          gtk_label_set_text (GTK_LABEL (document->priv->label), utf8_basename);
-
-          /* set the tab tooltip */
-          gtk_widget_set_tooltip_text (document->priv->ebox, utf8_filename);
-
-          /* update label color */
-          mousepad_document_label_color (document);
-        }
+      /* update label color */
+      mousepad_document_label_color (document);
     }
 }
 
@@ -704,6 +615,99 @@ mousepad_document_get_filename (MousepadDocument *document)
 
 
 
+static gboolean
+mousepad_document_search_completed_idle (gpointer data)
+{
+  MousepadDocument        *document = data;
+  MousepadSearchFlags      flags;
+  GtkSourceSearchContext  *search_context;
+  GtkSourceSearchSettings *search_settings;
+  GtkTextBuffer           *selection_buffer;
+  GtkTextIter             *start, *end;
+  GtkTextIter              iter;
+  gchar                   *selected_text;
+  const gchar             *string, *replace;
+  gboolean                 found;
+
+  /* retrieve the first stage data */
+  search_context = mousepad_object_get_data (document, "search-context");
+  flags = GPOINTER_TO_INT (mousepad_object_get_data (search_context, "flags"));
+  replace = mousepad_object_get_data (search_context, "replace");
+  start = mousepad_object_get_data (search_context, "start");
+  end = mousepad_object_get_data (search_context, "end");
+  found = GPOINTER_TO_INT (mousepad_object_get_data (search_context, "found"));
+  search_settings = gtk_source_search_context_get_settings (search_context);
+  string = gtk_source_search_settings_get_search_text (search_settings);
+
+  if (flags & MOUSEPAD_SEARCH_FLAGS_ITER_SEL_START)
+    gtk_text_buffer_get_selection_bounds (document->buffer, &iter, NULL);
+  else
+    gtk_text_buffer_get_selection_bounds (document->buffer, NULL, &iter);
+
+  /* handle the action */
+  if (found && (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT)
+      && ! (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION))
+    gtk_text_buffer_select_range (document->buffer, start, end);
+  else if (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE)
+    {
+      if (found && ! (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA))
+        {
+          /* replace selected occurrence */
+#if GTK_SOURCE_MAJOR_VERSION >= 4
+          gtk_source_search_context_replace (search_context, start, end, replace, -1, NULL);
+#else
+          gtk_source_search_context_replace2 (search_context, start, end, replace, -1, NULL);
+#endif
+
+          /* select next occurrence */
+          flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT;
+          flags &= ~ MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE;
+          mousepad_document_search (document, string, NULL, flags);
+        }
+      else if (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA)
+        {
+          /* replace all occurrences in the buffer in use */
+          gtk_source_search_context_replace_all (search_context, replace, -1, NULL);
+
+          if (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION)
+            {
+              /* get the text in the virtual selection buffer */
+              selection_buffer = GTK_TEXT_BUFFER (gtk_source_search_context_get_buffer (search_context));
+              gtk_text_buffer_get_bounds (selection_buffer, start, end);
+              selected_text = gtk_text_buffer_get_text (selection_buffer, start, end, FALSE);
+
+              /* replace selection in the real buffer by the text in the virtual selection buffer */
+              gtk_text_buffer_get_selection_bounds (document->buffer, start, end);
+              gtk_text_buffer_begin_user_action (document->buffer);
+              gtk_text_buffer_delete (document->buffer, start, end);
+              gtk_text_buffer_insert (document->buffer, start, selected_text, -1);
+              gtk_text_buffer_end_user_action (document->buffer);
+              g_free (selected_text);
+            }
+        }
+    }
+  /* deselect previous result when the new search fails or the search field is reset */
+  else if (! (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_NONE)
+           && ! (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION))
+    gtk_text_buffer_place_cursor (document->buffer, &iter);
+
+  document->priv->search_id = 0;
+
+  return FALSE;
+}
+
+
+
+static gboolean
+mousepad_document_unref (gpointer data)
+{
+  g_object_unref (data);
+
+  return FALSE;
+}
+
+
+
 static void
 mousepad_document_search_completed (GObject      *object,
                                     GAsyncResult *result,
@@ -712,12 +716,8 @@ mousepad_document_search_completed (GObject      *object,
   MousepadDocument        *document = data;
   MousepadSearchFlags      flags;
   GtkSourceSearchContext  *search_context = GTK_SOURCE_SEARCH_CONTEXT (object);
-  GtkSourceSearchSettings *search_settings;
-  GtkTextBuffer           *selection_buffer;
-  GtkTextIter              iter, start, end;
+  GtkTextIter              start, end;
   GError                  *error = NULL;
-  gchar                   *selected_text;
-  const gchar             *string, *replace;
   gboolean                 found;
 
   /* exit if the document was removed during the search process */
@@ -732,18 +732,8 @@ mousepad_document_search_completed (GObject      *object,
   /* remove extra reference kept at first stage */
   g_object_unref (document);
 
-  /* retrieve the first stage data */
-  flags = GPOINTER_TO_INT (mousepad_object_get_data (search_context, "flags"));
-  replace = mousepad_object_get_data (search_context, "replace");
-  search_settings = gtk_source_search_context_get_settings (search_context);
-  string = gtk_source_search_settings_get_search_text (search_settings);
-
-  if (flags & MOUSEPAD_SEARCH_FLAGS_ITER_SEL_START)
-    gtk_text_buffer_get_selection_bounds (document->buffer, &iter, NULL);
-  else
-    gtk_text_buffer_get_selection_bounds (document->buffer, NULL, &iter);
-
   /* get the search result */
+  flags = GPOINTER_TO_INT (mousepad_object_get_data (search_context, "flags"));
 #if GTK_SOURCE_MAJOR_VERSION >= 4
   if (flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD)
     found = gtk_source_search_context_backward_finish (search_context, result,
@@ -762,7 +752,15 @@ mousepad_document_search_completed (GObject      *object,
 
   /* exit if the operation was cancelled */
   if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-    return;
+    {
+      g_error_free (error);
+      return;
+    }
+  else if (error != NULL)
+    {
+      g_warning ("%s", error->message);
+      g_error_free (error);
+    }
 
   /* force the signal emission, to cover cases where Mousepad search settings change without
    * changing GtkSourceView settings (e.g. when switching between single-document mode and
@@ -770,53 +768,20 @@ mousepad_document_search_completed (GObject      *object,
   if (gtk_source_search_context_get_occurrences_count (search_context) != -1)
     g_object_notify (G_OBJECT (search_context), "occurrences-count");
 
-  /* handle the action */
-  if (found && (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT)
-      && ! (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION))
-    gtk_text_buffer_select_range (document->buffer, &start, &end);
-  else if (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE)
-    {
-      if (found && ! (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA))
-        {
-          /* replace selected occurrence */
-#if GTK_SOURCE_MAJOR_VERSION >= 4
-          gtk_source_search_context_replace (search_context, &start, &end, replace, -1, NULL);
-#else
-          gtk_source_search_context_replace2 (search_context, &start, &end, replace, -1, NULL);
-#endif
+  /* now we need first to exit this async task callback to prevent any warning,
+   * but we launch the rest as soon as possible to preserve real-time behavior */
+  mousepad_object_set_data (search_context, "found", GINT_TO_POINTER (found));
+  mousepad_object_set_data_full (search_context, "start",
+                                 gtk_text_iter_copy (&start), gtk_text_iter_free);
+  mousepad_object_set_data_full (search_context, "end",
+                                 gtk_text_iter_copy (&end), gtk_text_iter_free);
+  mousepad_object_set_data (document, "search-context", search_context);
+  if (document->priv->search_id != 0)
+    g_source_remove (document->priv->search_id);
 
-          /* select next occurrence */
-          flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT;
-          flags &= ~ MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE;
-          mousepad_document_search (document, string, NULL, flags);
-          return;
-        }
-      else if (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA)
-        {
-          /* replace all occurrences in the buffer in use */
-          gtk_source_search_context_replace_all (search_context, replace, -1, NULL);
-
-          if (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION)
-            {
-              /* get the text in the virtual selection buffer */
-              selection_buffer = GTK_TEXT_BUFFER (gtk_source_search_context_get_buffer (search_context));
-              gtk_text_buffer_get_bounds (selection_buffer, &start, &end);
-              selected_text = gtk_text_buffer_get_text (selection_buffer, &start, &end, FALSE);
-
-              /* replace selection in the real buffer by the text in the virtual selection buffer */
-              gtk_text_buffer_get_selection_bounds (document->buffer, &start, &end);
-              gtk_text_buffer_begin_user_action (document->buffer);
-              gtk_text_buffer_delete (document->buffer, &start, &end);
-              gtk_text_buffer_insert (document->buffer, &start, selected_text, -1);
-              gtk_text_buffer_end_user_action (document->buffer);
-              g_free (selected_text);
-            }
-        }
-    }
-  /* deselect previous result when the new search fails or the search field is reset */
-  else if (! (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_NONE)
-           && ! (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION))
-    gtk_text_buffer_place_cursor (document->buffer, &iter);
+  document->priv->search_id =
+    g_idle_add_full (G_PRIORITY_HIGH, mousepad_document_search_completed_idle,
+                     mousepad_util_source_autoremove (document), NULL);
 }
 
 
@@ -851,6 +816,9 @@ mousepad_document_search (MousepadDocument    *document,
           document->priv->selection_buffer = gtk_source_buffer_new (NULL);
           document->priv->selection_context =
             gtk_source_search_context_new (document->priv->selection_buffer, NULL);
+          g_signal_connect_swapped (document->priv->selection_context, "notify::occurrences-count",
+                                    G_CALLBACK (mousepad_document_emit_search_signal), document);
+          gtk_source_search_context_set_highlight (document->priv->selection_context, FALSE);
         }
 
       /* get the text in the selection area */
@@ -863,13 +831,8 @@ mousepad_document_search (MousepadDocument    *document,
       gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (document->priv->selection_buffer), &iter);
       g_free (selected_text);
 
-      /* associate the search context */
-      search_context = document->priv->selection_context;
-      gtk_source_search_context_set_highlight (search_context, FALSE);
-      g_signal_connect_swapped (search_context, "notify::occurrences-count",
-                                G_CALLBACK (mousepad_document_emit_search_signal), document);
-
       /* copy search settings from those of the document */
+      search_context = document->priv->selection_context;
       search_settings = gtk_source_search_context_get_settings (search_context);
       search_settings_doc = gtk_source_search_context_get_settings (document->priv->search_context);
 
@@ -996,6 +959,10 @@ mousepad_document_prevent_endless_scanning (MousepadDocument *document,
       mousepad_disconnect_by_func (document->buffer, mousepad_document_scanning_started, document);
       mousepad_disconnect_by_func (document->priv->search_context,
                                    mousepad_document_scanning_completed, document);
+
+      /* re-enable highlighting if disconnection occurs between the two switches above */
+      gtk_source_search_context_set_highlight (document->priv->search_context,
+                                               MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_HIGHLIGHT_ALL));
     }
 }
 
