@@ -471,7 +471,7 @@ mousepad_history_session_save (void)
     }
 
   /* allocate the session array */
-  session = g_malloc0 ((length + 1) * sizeof (gchar *));
+  session = g_new0 (gchar *, length + 1);
 
   /* fill in the session array */
   for (li = list, length = 0; li != NULL; li = li->next)
@@ -565,7 +565,7 @@ mousepad_history_session_restore (MousepadApplication *application)
         p++;
 
       /* allocate the GFile array */
-      files = g_malloc (n_uris * sizeof (GFile *));
+      files = g_new (GFile *, n_uris);
 
       /* add files from the session array */
       for (n = 0, current = 0, n_files = 0; n < n_uris; n++)
@@ -598,7 +598,7 @@ mousepad_history_session_restore (MousepadApplication *application)
               autosave_file = g_file_new_for_uri (autosave_uri);
 
               /* validate file */
-              if (! mousepad_util_validate_file (autosave_file))
+              if (mousepad_util_get_path (autosave_file) == NULL)
                 {
                   g_warning (CORRUPTED_SESSION_DATA);
                   g_object_unref (autosave_file);
@@ -631,27 +631,13 @@ mousepad_history_session_restore (MousepadApplication *application)
               uri++;
             }
 
-          /* see if there is a valid uri */
+          /* see if there is a uri */
           file = NULL;
           if (*uri != '\0')
-            {
               file = g_file_new_for_uri (uri);
-              if (! mousepad_util_validate_file (file))
-                {
-                  g_warning (CORRUPTED_SESSION_DATA);
-                  g_object_unref (file);
-                  if (autosave_file != NULL)
-                    g_object_unref (autosave_file);
-
-                  if (current == n_files)
-                    current = 0;
-
-                  continue;
-                }
-            }
 
           /* there is at least one autosaved file to restore */
-          if (! autosaved && autosave_file != NULL && g_file_query_exists (autosave_file, NULL))
+          if (! autosaved && autosave_file != NULL && mousepad_util_query_exists (autosave_file, FALSE))
             {
               autosaved = TRUE;
 
@@ -667,9 +653,9 @@ mousepad_history_session_restore (MousepadApplication *application)
             }
 
           /* add the file or drop it if it was removed since last session */
-          if (file != NULL && g_file_query_exists (file, NULL) && (
+          if (file != NULL && mousepad_util_query_exists (file, TRUE) && (
                 restore_setting != MOUSEPAD_SESSION_RESTORE_UNSAVED
-                || (autosave_file != NULL && g_file_query_exists (autosave_file, NULL))
+                || (autosave_file != NULL && mousepad_util_query_exists (autosave_file, FALSE))
               ))
             {
               mousepad_object_set_data_full (file, "autosave-uri", autosave_uri, g_free);
@@ -678,7 +664,7 @@ mousepad_history_session_restore (MousepadApplication *application)
                 g_object_unref (autosave_file);
             }
           else if (restore_autosaved && autosave_file != NULL
-                   && g_file_query_exists (autosave_file, NULL))
+                   && mousepad_util_query_exists (autosave_file, FALSE))
             {
               /* keep original uri if it exists */
               if (file != NULL)
