@@ -17,6 +17,8 @@
 #ifndef __MOUSEPAD_REPLACE_DIALOG_H__
 #define __MOUSEPAD_REPLACE_DIALOG_H__
 
+#include <mousepad/mousepad-window.h>
+
 #include <gtk/gtk.h>
 
 G_BEGIN_DECLS
@@ -33,13 +35,16 @@ typedef struct _MousepadReplaceDialog      MousepadReplaceDialog;
 
 GType           mousepad_replace_dialog_get_type       (void) G_GNUC_CONST;
 
-GtkWidget      *mousepad_replace_dialog_new            (void);
+GtkWidget      *mousepad_replace_dialog_new            (MousepadWindow        *window);
 
 void            mousepad_replace_dialog_history_clean  (void);
 
-void            mousepad_replace_dialog_page_switched  (MousepadReplaceDialog *dialog);
+void            mousepad_replace_dialog_page_switched  (MousepadReplaceDialog *dialog,
+                                                        GtkTextBuffer         *old_buffer,
+                                                        GtkTextBuffer         *new_buffer);
 
-void            mousepad_replace_dialog_set_text       (MousepadReplaceDialog *dialog, gchar *text);
+void            mousepad_replace_dialog_set_text       (MousepadReplaceDialog *dialog,
+                                                        const gchar           *text);
 
 G_END_DECLS
 
