@@ -22,13 +22,9 @@
 
 G_BEGIN_DECLS
 
-#define mousepad_util_validate_file(file) \
-  (mousepad_util_get_path (file) != NULL || ( \
-    g_file_has_uri_scheme (file, "trash") \
-    && g_file_query_exists (file, NULL) \
-    && g_file_query_file_type (file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL) \
-        != G_FILE_TYPE_SYMBOLIC_LINK \
-  ))
+#define mousepad_util_is_symlink(file) \
+  (g_file_query_file_type (file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL) \
+    == G_FILE_TYPE_SYMBOLIC_LINK)
 
 gboolean     mousepad_util_iter_inside_word                 (const GtkTextIter          *iter);
 
@@ -101,6 +97,11 @@ gchar       *mousepad_util_pango_font_description_to_css    (const PangoFontDesc
 
 const gchar *mousepad_util_get_path                         (GFile                      *file);
 
+gboolean     mousepad_util_query_exists                     (GFile                      *file,
+                                                             gboolean                    follow_symlink);
+
+gchar       *mousepad_util_get_display_path                 (GFile                      *file);
+
 gpointer     mousepad_util_source_autoremove                (gpointer                    object);
 
 G_END_DECLS
