@@ -29,6 +29,11 @@ G_BEGIN_DECLS
 #define MOUSEPAD_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MOUSEPAD_TYPE_WINDOW))
 #define MOUSEPAD_WINDOW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), MOUSEPAD_TYPE_WINDOW, MousepadWindowClass))
 
+/* sanity checks */
+#define mousepad_is_application_window(window) \
+  (g_list_find (gtk_application_get_windows (GTK_APPLICATION (g_application_get_default ())), \
+                window) != NULL)
+
 enum
 {
   TARGET_TEXT_URI_LIST,
@@ -67,6 +72,15 @@ void            mousepad_window_update_document_menu_items (MousepadWindow
 
 void            mousepad_window_update_window_menu_items   (MousepadWindow       *window);
 
+/* for plugins */
+GtkWidget      *mousepad_window_get_notebook               (MousepadWindow       *window);
+
+GtkWidget      *mousepad_window_menu_item_realign          (MousepadWindow       *window,
+                                                            GtkWidget            *item,
+                                                            const gchar          *action_name,
+                                                            GtkWidget            *menu,
+                                                            gint                  index);
+
 G_END_DECLS
 
 #endif /* !__MOUSEPAD_WINDOW_H__ */
